// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *	Reliable Broadcast over unreliable low-bandwidth (satellite)
 *			channels with no feedback
 *
 * 	Standard utility services used by many parts of the project
 *
 * $Id: util.cc,v 1.2 1996/08/05 19:55:00 oleg Exp oleg $
 *
 ************************************************************************
 */

#ifdef __GNUC__
#pragma implementation
#endif
#include "util.h"
#include "myenv.h"
#include "std.h"
#include <stdarg.h>
#include <signal.h>
#include <time.h>
extern "C"
{
#include <sys/types.h>
}
#ifndef __sun__
#define sigset(X,Y) signal(X,Y)
#endif

static const char Logger_env_name [] = "BC_LOG_FILE";

//------------------------------------------------------------------------
//			A logging service
//
// It filters and logs various system activities onto stderr, unless
// environment variable Logger_env_name specifies another file name.


			// Find out about the log file, and open it
Logger::Logger(void)
{
  const char * log_file_name = xgetenv(Logger_env_name,"");
  if( log_file_name[0] != '\0' )
  {
    if( (log_fp = fopen(log_file_name,"w")) == 0 )
      perror("Log file open error"),
      _error("Failed to open the log file '%s' because of the error above",
      	     log_file_name);
  }
  else
    log_fp = stderr;
}

Logger Logger::me;	// this is the only instance of logger that can be made
			// The constructor above would be called on 'me'
			// before main()

			// Close the log
Logger::~Logger(void)
{
  log("The log is closed");
  if( log_fp != stderr )
    fclose(log_fp);
}

			// Log a message (possibly followed by
			// arguments, in a manner similar to printf())
void Logger::log(const char * message,...)
{
  va_list args;
  va_start(args,message);               // Init 'args' to the beginning of
                                        // the variable length list of args
  time_t clock_val = time(0);
  char buffer[60]; buffer[0] = '\0';
  strftime(buffer,sizeof(buffer)-1,"%c",localtime(&clock_val));

  fprintf(me.log_fp,"\[%s] ",buffer);
  vfprintf(me.log_fp,message,args);
  fputs("\n",me.log_fp);
  fflush(me.log_fp);
}

//------------------------------------------------------------------------
//			Handling Signals
//
// At present, no nested handlers are allowed. Otherwise, Signal::curr_Signal
// would be a stack of <Signal *> rather than a simple pointer. Instantiating
// a Signal would push then the current object onto the stack, and destructor
// would pop it back.

Signal * Signal::curr_Signal = 0;

			// Install our dispatch_relay and instantiate the
			// current Signal instance
			// Note, if your OS doesn't have sigset(), use
			// signal() instead.
Signal::Signal(const int _signal_id)
	: signal_id(_signal_id),
	  prev_disposition(sigset(_signal_id,disposition_relay))
{
  if( prev_disposition == SIG_ERR )
    perror("setting the signal"),
    _error("Failed to set up the handler for the signal %d",signal_id);
  if( prev_disposition == disposition_relay )
    _error("Nesting of Signals isn't allowed, yet");
  curr_Signal = this;
}


			// Uninstall the signal
Signal::~Signal(void)
{
  if( sigset(signal_id,prev_disposition) == SIG_ERR )
    perror("resetting the signal"),
    _error("Error restoring the disposition for signal %d",signal_id);
  curr_Signal = 0;
}

			// Relaying a signal to the hieararchy of Signals
void Signal::disposition_relay(int signal_id)
{
  assert( curr_Signal != 0 );
  curr_Signal->handle();	// Disregard handle's return code at the moment
}

//------------------------------------------------------------------------
//			Random number generating service

				// Initialize the random number generator
Random::Random(void)
{
  srand(time(0));
}

Random Random::me;	// this is the only instance of Random that can be made
			// The constructor above would be called on 'me'
			// before main()


